<?php
namespace InspireLabs\WooInpost\shipx\services\shipment;

use InspireLabs\WooInpost\EasyPack;
use Exception;
use InspireLabs\WooInpost\EasyPack_API;
use InspireLabs\WooInpost\shipping\EasyPack_Shipping_Method_Courier;
use InspireLabs\WooInpost\shipping\EasyPack_Shippng_Parcel_Machines;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Model;
use InspireLabs\WooInpost\shipx\models\shipment_cost\ShipX_Shipment_Cost_Model;
use ReflectionClass;
use ReflectionException;

class ShipX_Shipment_Price_Calculator_Service
{

    /**
     * @var EasyPack_API
     */
    private $api;

    public function __construct()
    {
        $this->api = EasyPack_API();
    }

    /**
     * @param array $shipment
     *
     *
     * @return array
     * @throws \Exception
     */
    private function do_calculate_shipment($shipment)
    {
        return $this->api->calculate_shipments(['shipments' => $shipment]);
    }


    /**
     * @throws ReflectionException
     * @throws Exception
     */
    public function calculate_shipment_cost_ajax()
    {
        $order_id = (int)$_POST['order_id'];
        $shipment = $this->get_shipment_by_order_id($order_id);

        try {
            $response = $this->calculate_created_shipment_cost($shipment,
                $order_id);
            $costs = $this->map_cost_collection($response);
        } catch (Exception $e) {
            echo json_encode([
                'status' => 'error',
                'message' => $e->getMessage(),
            ], JSON_FORCE_OBJECT);
            exit;
        }

        echo json_encode([
            'status' => 'ok',
            'cost' => $this->dismount($costs[0]),
        ], JSON_FORCE_OBJECT);
        exit;

    }


    /**
     * @param ShipX_Shipment_Model $shipment
     *
     * @return ShipX_Shipment_Cost_Model
     * @throws ReflectionException
     */
    public function get_calculated_shipment_cost($shipment)
    {
        if (null !== $shipment) {
            $response = $this->calculate_created_shipment_cost($shipment);
        } else {
            $response = $this->calculate_new_shipment_cost();
        }
        $costs = $this->map_cost_collection($response);

        return $costs[0];
    }

    /**
     * @param array $costs
     *
     * @return ShipX_Shipment_Cost_Model[]
     */
    private function map_cost_collection($costs)
    {
        $cost_collection = [];
        foreach ($costs as $cost) {
            $cost_model = new ShipX_Shipment_Cost_Model();
            if ($this->is_error($cost)) {
                $cost_model->setError(true);
                $cost_model->setId($cost['id']);
                $cost_model->setErrorKey($cost['key']);
                $cost_model->setErrorMessage($cost['message']);
            } else {
                $cost_model->setId($cost['id']);
                $cost_model->setCalculatedChargeAmount($cost['calculated_charge_amount']);
                $cost_model->setFuelChargeAmount($cost['fuel_charge_amount']);
                $cost_model->setNotificationChargeAmount($cost['notification_charge_amount']);
                $cost_model->setCodChargeAmount($cost['cod_charge_amount']);
                $cost_model->setInsuranceChargeAmount($cost['insurance_charge_amount']);
                $cost_model->setCalculatedChargeAmountNonCommission($cost['calculated_charge_amount_non_commission']);
            }
            $cost_collection[] = $cost_model;
        }

        return $cost_collection;
    }

    /**
     * @param array $cost
     *
     * @return bool
     */
    private function is_error($cost)
    {
        return isset($cost['key']);
    }


    /**
     * @throws ReflectionException
     * @throws Exception
     */
    public function calculate_new_shipment_cost()
    {
        $shipment_model
            = EasyPack_Shippng_Parcel_Machines::ajax_create_shipment_model();
        $shipment_service = EasyPack::EasyPack()->get_shipment_service();
        $shipment_array = $shipment_service->shipment_to_array($shipment_model);
        $response = $this->do_calculate_shipment($shipment_array);

        return $response;
    }

    /**
     * @param ShipX_Shipment_Model $shipment_model_from_db
     *
     * @param                      $order_id
     *
     * @return mixed
     * @throws Exception
     */
    public function calculate_created_shipment_cost(
        $shipment_model_from_db = null,
        $order_id
    ) {
        $order = wc_get_order($order_id);
        $shipping_methods = $order->get_shipping_methods();
        $shipping_method = @array_shift($shipping_methods);
        $shipping_method_id = $shipping_method['method_id'];

        $is_courier_service = false;
        if (strpos($shipping_method_id, 'courier') !== false) {
            $is_courier_service = true;
        }

        if ($shipment_model_from_db instanceof ShipX_Shipment_Model) {
            $shipment_service = EasyPack::EasyPack()->get_shipment_service();
            $shipment_array
                = $shipment_service->shipment_to_array($shipment_model_from_db);
            $response = $this->do_calculate_shipment($shipment_array);
        } else {
            if (true === $is_courier_service) {
                $shipment_model_from_post
                    = EasyPack_Shipping_Method_Courier::ajax_create_shipment_model();
            } else {
                $shipment_model_from_post
                    = EasyPack_Shippng_Parcel_Machines::ajax_create_shipment_model();
            }

            $shipment_service = EasyPack::EasyPack()->get_shipment_service();
            $shipment_array
                = $shipment_service->shipment_to_array($shipment_model_from_post);
            $response = $this->do_calculate_shipment($shipment_array);
        }


        return $response;
    }


    /**
     * @param $order_id
     *
     * @return ShipX_Shipment_Model|null
     */
    public function get_shipment_by_order_id($order_id)
    {
        $order = wc_get_order($order_id);
        $from_order_meta = $order->get_meta('_shipx_shipment_object');

        return $from_order_meta instanceof ShipX_Shipment_Model
            ? $from_order_meta
            : null;
    }

    function dismount($object)
    {
        $reflectionClass = new ReflectionClass(get_class($object));
        $array = [];
        foreach ($reflectionClass->getProperties() as $property) {
            $property->setAccessible(true);
            $array[$property->getName()] = $property->getValue($object);
            $property->setAccessible(false);
        }

        return $array;
    }
}
